/**
 * Copyright (c) 2017-2018 INCHRON GmbH
 * 
 * This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 2.0
 * which is available at https://www.eclipse.org/legal/epl-2.0/
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *     INCHRON GmbH - Meta Model design and implementation
 */
package com.inchron.realtime.root.model;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import org.eclipse.emf.common.util.Enumerator;

/**
 * <!-- begin-user-doc -->
 * A representation of the literals of the enumeration '<em><b>Variable Read Access Policy</b></em>',
 * and utility methods for working with them.
 * <!-- end-user-doc -->
 * @see com.inchron.realtime.root.model.ModelPackage#getVariableReadAccessPolicy()
 * @model
 * @generated
 */
public enum VariableReadAccessPolicy implements Enumerator {
	/**
	 * The '<em><b>FIFO Read</b></em>' literal object.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @see #FIFO_READ_VALUE
	 * @generated
	 * @ordered
	 */
	FIFO_READ(0, "FIFORead", "FIFORead"), /**
	 * The '<em><b>FIFO Take</b></em>' literal object.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @see #FIFO_TAKE_VALUE
	 * @generated
	 * @ordered
	 */
	FIFO_TAKE(1, "FIFOTake", "FIFOTake"), /**
	 * The '<em><b>LIFO Read</b></em>' literal object.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @see #LIFO_READ_VALUE
	 * @generated
	 * @ordered
	 */
	LIFO_READ(2, "LIFORead", "LIFORead"), /**
	 * The '<em><b>LIFO Take</b></em>' literal object.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @see #LIFO_TAKE_VALUE
	 * @generated
	 * @ordered
	 */
	LIFO_TAKE(3, "LIFOTake", "LIFOTake");

	/**
	 * The '<em><b>FIFO Read</b></em>' literal value.
	 * <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of '<em><b>FIFO Read</b></em>' literal object isn't clear,
	 * there really should be more of a description here...
	 * </p>
	 * <!-- end-user-doc -->
	 * @see #FIFO_READ
	 * @model name="FIFORead"
	 * @generated
	 * @ordered
	 */
	public static final int FIFO_READ_VALUE = 0;

	/**
	 * The '<em><b>FIFO Take</b></em>' literal value.
	 * <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of '<em><b>FIFO Take</b></em>' literal object isn't clear,
	 * there really should be more of a description here...
	 * </p>
	 * <!-- end-user-doc -->
	 * @see #FIFO_TAKE
	 * @model name="FIFOTake"
	 * @generated
	 * @ordered
	 */
	public static final int FIFO_TAKE_VALUE = 1;

	/**
	 * The '<em><b>LIFO Read</b></em>' literal value.
	 * <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of '<em><b>LIFO Read</b></em>' literal object isn't clear,
	 * there really should be more of a description here...
	 * </p>
	 * <!-- end-user-doc -->
	 * @see #LIFO_READ
	 * @model name="LIFORead"
	 * @generated
	 * @ordered
	 */
	public static final int LIFO_READ_VALUE = 2;

	/**
	 * The '<em><b>LIFO Take</b></em>' literal value.
	 * <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of '<em><b>LIFO Take</b></em>' literal object isn't clear,
	 * there really should be more of a description here...
	 * </p>
	 * <!-- end-user-doc -->
	 * @see #LIFO_TAKE
	 * @model name="LIFOTake"
	 * @generated
	 * @ordered
	 */
	public static final int LIFO_TAKE_VALUE = 3;

	/**
	 * An array of all the '<em><b>Variable Read Access Policy</b></em>' enumerators.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private static final VariableReadAccessPolicy[] VALUES_ARRAY =
		new VariableReadAccessPolicy[] {
			FIFO_READ,
			FIFO_TAKE,
			LIFO_READ,
			LIFO_TAKE,
		};

	/**
	 * A public read-only list of all the '<em><b>Variable Read Access Policy</b></em>' enumerators.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public static final List<VariableReadAccessPolicy> VALUES = Collections.unmodifiableList(Arrays.asList(VALUES_ARRAY));

	/**
	 * Returns the '<em><b>Variable Read Access Policy</b></em>' literal with the specified literal value.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @param literal the literal.
	 * @return the matching enumerator or <code>null</code>.
	 * @generated
	 */
	public static VariableReadAccessPolicy get(String literal) {
		for (int i = 0; i < VALUES_ARRAY.length; ++i) {
			VariableReadAccessPolicy result = VALUES_ARRAY[i];
			if (result.toString().equals(literal)) {
				return result;
			}
		}
		return null;
	}

	/**
	 * Returns the '<em><b>Variable Read Access Policy</b></em>' literal with the specified name.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @param name the name.
	 * @return the matching enumerator or <code>null</code>.
	 * @generated
	 */
	public static VariableReadAccessPolicy getByName(String name) {
		for (int i = 0; i < VALUES_ARRAY.length; ++i) {
			VariableReadAccessPolicy result = VALUES_ARRAY[i];
			if (result.getName().equals(name)) {
				return result;
			}
		}
		return null;
	}

	/**
	 * Returns the '<em><b>Variable Read Access Policy</b></em>' literal with the specified integer value.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @param value the integer value.
	 * @return the matching enumerator or <code>null</code>.
	 * @generated
	 */
	public static VariableReadAccessPolicy get(int value) {
		switch (value) {
			case FIFO_READ_VALUE: return FIFO_READ;
			case FIFO_TAKE_VALUE: return FIFO_TAKE;
			case LIFO_READ_VALUE: return LIFO_READ;
			case LIFO_TAKE_VALUE: return LIFO_TAKE;
		}
		return null;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private final int value;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private final String name;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private final String literal;

	/**
	 * Only this class can construct instances.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private VariableReadAccessPolicy(int value, String name, String literal) {
		this.value = value;
		this.name = name;
		this.literal = literal;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public int getValue() {
	  return value;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public String getName() {
	  return name;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public String getLiteral() {
	  return literal;
	}

	/**
	 * Returns the literal value of the enumerator, which is its string representation.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public String toString() {
		return literal;
	}
	
} //VariableReadAccessPolicy
