/**
 ********************************************************************************
 * Copyright (c) 2018-2019 Robert Bosch GmbH.
 * 
 * This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 2.0
 * which is available at https://www.eclipse.org/legal/epl-2.0/
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 * Contributors:
 *     Robert Bosch GmbH - initial API and implementation
 ********************************************************************************
 */

package templates.utils;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.eclipse.app4mc.amalthea.model.AmaltheaCrossReferenceAdapter;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature.Setting;

public class AmaltheaModelNagivationUtils {

	public  static synchronized AmaltheaCrossReferenceAdapter getOrCreateAmaltheaAdapter(final Notifier target) {
		// Try to get Amalthea adapter
		final EList<Adapter> adapters = target.eAdapters();
		for (final Adapter adapter : adapters) {
			if (adapter instanceof AmaltheaCrossReferenceAdapter) {
				return (AmaltheaCrossReferenceAdapter) adapter;
			}
		}
		
		// Create Amalthea adapter
		final AmaltheaCrossReferenceAdapter amaltheaAdapter = new AmaltheaCrossReferenceAdapter();
		adapters.add(amaltheaAdapter);
		return amaltheaAdapter;
	}

	 public static Collection<EObject> getBackReferences(EObject eObject, boolean resolve){
		 
		 List<EObject> result=new ArrayList<EObject>();
		 
		 Collection<Setting> references = getOrCreateAmaltheaAdapter(eObject).getNonNavigableInverseReferences(eObject, resolve);
		 
		 for (Setting setting : references) {
			EObject eObject2 = setting.getEObject();
			
			if(result.contains(eObject2)==false) {
				result.add(eObject2);
			}
		}
		 return result;
		 
	 }
	 
	 
}
