/**
 ********************************************************************************
 * Copyright (c) 2018-2020 Robert Bosch GmbH.
 * 
 * This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 2.0
 * which is available at https://www.eclipse.org/legal/epl-2.0/
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 * Contributors:
 *     Robert Bosch GmbH - initial API and implementation
 ********************************************************************************
 */

package org.eclipse.app4mc.transformation.application.base;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Properties;

import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.FileAppender;
import org.apache.log4j.Level;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;
import org.eclipse.equinox.app.IApplication;
import org.eclipse.equinox.app.IApplicationContext;

/**
 * This class controls all aspects of the application's execution
 */
public class Application implements IApplication {

	@Override
	public Object start(IApplicationContext context) throws Exception {

		Properties inputParameters = getInputParameters(context);

		if (inputParameters != null) {

			Logger logger = getLogger(inputParameters);

			logger.info("Starting Model transformation ...");

			ExecuteTransformation.start(logger, inputParameters);

			logger.removeAllAppenders();
		} else {
			System.out.println(
					"ERROR !! Unable to start transformation as required parameters are not set in input properties file");

			return Integer.valueOf(-1);
		}

		return IApplication.EXIT_OK;
	}

	protected Logger getLogger(Properties inputParameters) {

		String logFilePath = inputParameters.getProperty("log_file");

		org.apache.log4j.Logger logger = LogManager.getLogger("org.eclipse.app4mc.transformation");

		logger.setLevel(Level.INFO);
		try {
			logger.removeAllAppenders();
			/*- If required use the following options in pattern layout %d %-5p %t %c (%F:%L) %m" */
			logger.addAppender(
					new FileAppender(new PatternLayout("%d{yyyy-MM-dd_HH_mm_ss} - %-5p:  %m%n"), logFilePath, false));

			logger.addAppender(new ConsoleAppender(new PatternLayout("%d{yyyy-MM-dd_HH_mm_ss} - %-5p:  %m%n")));

		} catch (IOException e) {

			System.out.println("invalid log file path " + logFilePath + " specified in the input.properties file ");
			e.printStackTrace();
		}

		return logger;
	}

	protected Properties getInputParameters(IApplicationContext context) throws IOException, FileNotFoundException {

		String property = System.getProperty("user.dir");

		System.out.println(property);
		String[] args = (String[]) context.getArguments().get("application.args");

		if (args != null && args.length > 0) {

			String inputPropsFile = args[1];

			File file = new File(inputPropsFile);

			Properties properties = new Properties();

			properties.load(new FileInputStream(file));

			return properties;
		}

		return null;
	}

	@Override
	public void stop() {
		// nothing to do
	}
}
